﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.Win32;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;
using EnvDTE;
using System.IO;
using System.Text;

namespace DanielHughes.ForGetVisualStudioPackage
{
   /// <summary>
   /// This is the class that implements the package exposed by this assembly.
   ///
   /// The minimum requirement for a class to be considered a valid package for Visual Studio
   /// is to implement the IVsPackage interface and register itself with the shell.
   /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
   /// to do it: it derives from the Package class that provides the implementation of the 
   /// IVsPackage interface and uses the registration attributes defined in the framework to 
   /// register itself and its components with the shell.
   /// </summary>
   // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
   // a package.
   [PackageRegistration(UseManagedResourcesOnly = true)]
   // This attribute is used to register the information needed to show this package
   // in the Help/About dialog of Visual Studio.
   [InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)]
   [Guid(GuidList.guidForGetVisualStudioPackagePkgString)]
   [ProvideAutoLoad("ADFC4E64-0397-11D1-9F4E-00A0C911004F")]
   public sealed class ForGetVisualStudioPackagePackage : Package
   {
      /// <summary>
      /// Default constructor of the package.
      /// Inside this method you can place any initialization code that does not require 
      /// any Visual Studio service because at this point the package object is created but 
      /// not sited yet inside Visual Studio environment. The place to do all the other 
      /// initialization is the Initialize method.
      /// </summary>
      public ForGetVisualStudioPackagePackage()
      {
         Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering constructor for: {0}", this.ToString()));
      }

      private BuildEvents _buildEvents = null;

      /////////////////////////////////////////////////////////////////////////////
      // Overridden Package Implementation
      #region Package Members

      /// <summary>
      /// Initialization of the package; this method is called right after the package is sited, so this is the place
      /// where you can put all the initialization code that rely on services provided by VisualStudio.
      /// </summary>
      protected override void Initialize()
      {
         Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering Initialize() of: {0}", this.ToString()));
         base.Initialize();
         IServiceContainer serviceContainer = this as IServiceContainer;
         DTE sdte = serviceContainer.GetService(typeof(SDTE)) as DTE;
         _buildEvents = sdte.Events.BuildEvents;
         _buildEvents.OnBuildBegin += BuildEvents_OnBuildBegin;
      }

      private void BuildEvents_OnBuildBegin(vsBuildScope buildScope, vsBuildAction buildAction)
      {
         if (buildAction != vsBuildAction.vsBuildActionBuild && buildAction != vsBuildAction.vsBuildActionRebuildAll)
         {
            return;
         }

         Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Build Begin", this.ToString()));
         DTE dte = (DTE)GetService(typeof(DTE));
         string solutionDir = System.IO.Path.GetDirectoryName(dte.Solution.FullName);
         string depedenciesFile = Path.Combine(solutionDir, "dependencies.config");
         if(!File.Exists(depedenciesFile))
         {
            return; //this solution is not using ForGet
         }

         using (var process = new System.Diagnostics.Process())
         {
            process.StartInfo = new ProcessStartInfo()
            {
               FileName = "ForGet",
               Arguments = "-r",
               WorkingDirectory = solutionDir,
               RedirectStandardError = true,
               RedirectStandardOutput = true,
               UseShellExecute = false,
               CreateNoWindow = true,
            };

            process.OutputDataReceived += ProcessDataReceived;
            process.ErrorDataReceived += ProcessDataReceived;
            if (!process.Start())
            {
               Debug.WriteLine("ERROR: Failed to start ForGet");
               return;
            }
            process.BeginOutputReadLine();
            process.BeginErrorReadLine();
            process.WaitForExit();
            if (process.ExitCode != 0)
            {
               Console.Error.WriteLine("ERROR: ForGet.exe failed");
               return;
            }
            return;
         }
      }

      void ProcessDataReceived(object sender, DataReceivedEventArgs e)
      {
         if (e.Data != null)
         {
            if (e.Data.Length != 0)
            {
               string buildOutput = string.Format("ForGet: {0}{1}", e.Data, Environment.NewLine);
               OutputString(Microsoft.VisualStudio.VSConstants.OutputWindowPaneGuid.BuildOutputPane_guid, buildOutput);
            }
         }
      }

      void OutputString(Guid guidPane, string text)
      {
         const int VISIBLE = 1;
         const int DO_NOT_CLEAR_WITH_SOLUTION = 0;

         IVsOutputWindow outputWindow;
         IVsOutputWindowPane outputWindowPane = null;
         int hr;

         // Get the output window
         outputWindow = base.GetService(typeof(SVsOutputWindow)) as IVsOutputWindow;

         // The General pane is not created by default. We must force its creation
         if (guidPane == Microsoft.VisualStudio.VSConstants.OutputWindowPaneGuid.GeneralPane_guid)
         {
            hr = outputWindow.CreatePane(guidPane, "General", VISIBLE, DO_NOT_CLEAR_WITH_SOLUTION);
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(hr);
         }

         // Get the pane
         hr = outputWindow.GetPane(guidPane, out outputWindowPane);
         Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(hr);

         // Output the text
         if (outputWindowPane != null)
         {
            outputWindowPane.Activate();
            outputWindowPane.OutputString(text);
         }
      }
      #endregion

   }
}
